#include "htkGraph.h"
#include "graph.h"
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <sstream>
#include <fstream>

/* modif FRED: Octobre 2008 - reading 'start' and 'end' states in the HTK file */

using namespace std;


static string IntToStr(int i)
{
	ostringstream oss;
	oss << i;
	return oss.str();
}

static int StrToInt(string s)
{
	istringstream iss( s );
	int n;
	iss >> n;
	return n;
}

static double StrToDouble(string s)
{
	istringstream iss( s );
	double n;
	iss >> n;
	return n;
}

static string DoubleToStr(double f)
{
	ostringstream oss;
	oss << f;
	return oss.str();
}

// check if the number n is in the given path
static bool isInPath(int* path, int length, int n)
{
	for(int i=0;i<length;i++)
		if (path[i]==n)
			return true;
	
	return false;
}



//FIXME: CRAPPY METHOD : Doesn't work with recursivity !!!!
static string XMLFindAttr(xmlNode* root_node, unsigned int& index, const XMLAttrPos& xml_attr_pos)
{
	xmlNode* cur_node = NULL;

	int num_elem=0;
	for(cur_node = root_node; cur_node; cur_node = cur_node->next) 
	{
		num_elem++;
		if (num_elem == xml_attr_pos.attr_pos[index])
		{
			if (index == xml_attr_pos.attr_pos.size()-1)
			{
				if (cur_node->properties != NULL)
				{
					xmlAttr* attr;
					for(attr = cur_node->properties; attr; attr = attr->next)
					{
						//printf("ATTRIBUTE NAME: %s\n", attr->name);
						//printf("ATTRIBUTE VAL: %s\n", attr->children->content);
						string name((char*)attr->name);
						if (name == xml_attr_pos.attr_name)
							return ((char*)attr->children->content);
					}
					return "";
				}
			}
			else
				return "";
			
			if (cur_node->children)
			{
				index++;
				return XMLFindAttr(cur_node->children, index, xml_attr_pos);
			}
		}
	}
	
	return "";
}


//TODO: Check the recursivity for this one
static string XMLFindAttr(xmlNode* root_node, unsigned int& index, const std::vector<int>& path)
{
	xmlNode* cur_node = NULL;
	index++;
	int num_elem=0;
	for(cur_node = root_node; cur_node; cur_node = cur_node->next)
	{
		if (index == path.size()-1)
		{
			if (cur_node->properties != NULL)
			{
				xmlAttr* attr;
				for(attr = cur_node->properties; attr; attr = attr->next)
				{
					if (path[index] == num_elem)
						return ((char*)attr->children->content);
					num_elem++;
				}
				return "";
			}
		}
		
		
		if (cur_node->children)
		{
			string result(XMLFindAttr(cur_node->children, index, path));
			if (result!="") return result;
		}
		
		num_elem++;	
	}
	return "";
}



static string XMLGetFirstAttr(xmlNode* root_node)
{
	xmlNode* cur_node = NULL;

	for(cur_node = root_node; cur_node; cur_node = cur_node->next) 
	{
		if (cur_node->properties != NULL)
		{
			xmlAttr* attr;
			for(attr = cur_node->properties; attr; attr = attr->next)
			{
				//printf("ATTRIBUTE NAME: %s\n", attr->name);
				//printf("ATTRIBUTE VAL: %s\n", attr->children->content);
				return ((char*)attr->children->content);
			}
		}
		
		if (cur_node->children)
			return XMLGetFirstAttr(cur_node->children);
	}
	
	return "";
}




HTKGraph::HTKGraph()
{
	graph = NULL;
	nodes = NULL;
	display_shortest_path = false;
	distance_shortest_path = -1;
	distance_shortest_path_by_an_edge = -1;
	display_shortest_path = false;
	display_shortest_path_by_an_edge = false;
	edge_selected.id = -1;
	edge_selected.x = -1;
	edge_selected.y = -1;
	information_available = false;
	display_score = false;
	score_opposite = true;
}


HTKGraph::~HTKGraph()
{
	delete graph;
	delete [] nodes;
}


void HTKGraph::displayShortestPath(bool display)
{
	display_shortest_path = display;
	if (!display_shortest_path)
		distance_shortest_path = -1;
}


void HTKGraph::displayShortestPathByAnEdge(bool display, const GraphElement& _edge_selected)
{
	display_shortest_path_by_an_edge = display; 
	if (!display)
		distance_shortest_path_by_an_edge = -1;
	
	edge_selected = _edge_selected;
}

double HTKGraph::getDistance()
{
	return distance_shortest_path;
}

double HTKGraph::getDistanceByAnEdge()
{
	return distance_shortest_path_by_an_edge;
}


void HTKGraph::takeTheOppositeScore(bool opposite, std::list<GraphElement>& graph_elements)
{
	if ((!graph) || (!nodes)) 
	{
		cout << "WARNING : takeTheOppositeScore called but either graph or nodes is NULL" << endl;
		return;
	}

	if (opposite == score_opposite) return;
	
	// if we are here we have to compute the opposite
	score_opposite = opposite;
	
	std::list<GraphElement>::iterator it;
	for(it = graph_elements.begin(); it != graph_elements.end(); it++)
		(*it).value = -(*it).value;
	
	int n_nodes = graph->getNbSommets();
	
	map<string,double>::iterator iter;
	for(int i=0;i<n_nodes;i++)
		for( iter = nodes[i].values.begin(); iter != nodes[i].values.end(); iter++ ) 
			iter->second = -iter->second;
		
	Arc** S = graph->getSommet();
	for (int i=0;i<graph->getNbSommets();i++)
		for(Arc* pt=S[i];pt;pt=pt->suivant()) // for each edge
			pt->takeTheOppositeScore();

}

string HTKGraph::getXMLInfo(const string& xml, bool use_path)
{
	xmlDocPtr doc; // the resulting document tree

	doc = xmlReadMemory(xml.c_str(), xml.size(), "getXMLInfo.xml", NULL, 0);
	if (doc == NULL) 
	{
		cout << "Failed to parse document" << endl;
		return "NULL";
	}
	
	// Get the root element node
	xmlNode* root_element = xmlDocGetRootElement(doc);
		
	string result;
	
	if (use_path?xml_path.empty():xml_attr.empty()) 
		result = XMLGetFirstAttr(root_element);
	else
	{
		unsigned int index = 0;
		use_path?result = XMLFindAttr(root_element, index, xml_path[0]):result = XMLFindAttr(root_element, index, xml_attr[0]);
		for(unsigned int i=1;i<(use_path?xml_path.size():xml_attr.size());i++)
		{
			index = 0;
			string tmp_result;
			use_path?tmp_result = XMLFindAttr(root_element, index, xml_path[i]):tmp_result = XMLFindAttr(root_element, index, xml_attr[i]);
			
			if (tmp_result != "")
			{
				if (result != "")
					result = result + " / " + tmp_result;
				else
					result += tmp_result;
			}
			
		}
		
		if (result == "")
			result = XMLGetFirstAttr(root_element);
	}
			
	if (result == "")
		result = "NULL"; // shouldn't happen but we _have_ to write something on each edge
	
	xmlFreeDoc(doc);
	xmlCleanupParser();
	
	return result;
}

string HTKGraph::parseHTK(const string& filename)
{
static char chfred[2000];
int i_start,i_end,ii;

	ifstream file(filename.c_str());
	if(!file)
		return false;

	/*................................*/
	/* FRED: read 'start' and 'end' nodes */
   {
   FILE *filefred;
   if (!(filefred=fopen(filename.c_str(),"rt"))) { cerr << "ERROR: can't open: " << filename << endl; exit(1); }
   i_start=i_end=-1;
    while (fgets(chfred,2000,filefred))
		if (sscanf(chfred,"start=%d",&ii)) i_start=ii;
	    else if (sscanf(chfred,"end=%d",&ii)) i_end=ii;
   fclose(filefred);
   }
	/*................................*/

	// parse all lines from the configuration file
	for(int line = 1; ; line++)
	{
		// read the next configuration line
		string strBuffer;
		getline(file, strBuffer);

		// stop if we reached the end of file
		if(file.eof()) break;

		// check if an error happend while reading from the file
		if(!file)
			return filename + " : file not found or permission denied";

		string::size_type pos=0;
		
		int id = 0,  from=1, dest=1;
		string XMLData;
		char line_type = 0;
		std::map<std::string, double> values;
		
		do
		{
			// find the first non-whitespace character
			pos = strBuffer.find_first_not_of(" \t", pos);
	
			// check for empty lines
			if((pos == string::npos) || (strBuffer[pos] == '\n') || (strBuffer[pos] == '\r') || (strBuffer[pos] == 0))
				break;
	
			// check for comment lines
			if(strBuffer[pos] == '#') break;
	
			// get the key
			string strKey;
			strKey = strBuffer.substr(pos, strBuffer.find_first_of(" =\t\n\r", pos) - pos);
			pos += strKey.size();
			
			if (strKey == "VERSION" || strKey == "version" || 
				strKey == "UTTERANCE" || strKey == "utterance" || 
				strKey == "LMSCALE" || strKey == "lmscale" || 
				strKey =="WDPENALTY" || strKey =="wdpenalty" || 
				strKey =="CONTEXT" || strKey =="context")
				break;
			
			// get the '=' character
			pos = strBuffer.find_first_not_of(" \t", pos);
			if((pos == string::npos) || (strBuffer[pos] != '='))
				return filename + ": syntax error line " + IntToStr(line);
	
			// find the first non-whitespace character after the '=' character
			pos = strBuffer.find_first_not_of(" \t", pos + 1);
	
			// get the data
			std::string strData;
			
			bool XML_field = (strKey == "W" || strKey == "WORD" || strKey == "word");
			
			if (!XML_field)
			{
				// we do not know this key so let's assume it's a value for either an edge or a node
				strData = strBuffer.substr(pos, strBuffer.find_first_of("\t\n\r #", pos) - pos);
				pos += strData.size();
			}

			if (XML_field)
			{
				unsigned int cpos=pos;
				int opened = 0;
				
				while(true)
				{
					if (strBuffer[pos]=='"')
					{
						cpos++;
						
						if (cpos > strBuffer.size())
							return filename + ": syntax error line " + IntToStr(line);
						
						if (strBuffer[cpos]=='\\')
						{
							cpos++;
						}
						else if (strBuffer[cpos]=='"')
						{
							pos = cpos+1;
							break;
						}
						
						strData+=strBuffer[cpos];
					}
					else
					if (strBuffer[pos]=='<')
					{
						if (cpos > strBuffer.size())
							return filename + ": syntax error line " + IntToStr(line);
						
						if (strBuffer[cpos]=='<')
							opened++;
						if (strBuffer[cpos]=='>')
							opened--;
						
						strData+=strBuffer[cpos];
						cpos++;
						
						if (opened == 0)
						{
							// HACK: stop parsing only if there is not any < until the end of the line
							// If there is one, parsing continue
							if ((strBuffer.find_first_of("<", cpos))==string::npos)
							{
								pos = cpos;
								break;
							}
						}
						
					}
					else
						return filename + ": syntax error line " + IntToStr(line);
					
				}
				XMLData = strData;
			}
			else if(strKey == "I")
			{
				line_type = 'I';
				id = StrToInt(strData);
			}
			else if (strKey == "t" || strKey == "TIME" || strKey == "time")
			{
				if (line_type == 'I')
					nodes[id].time = StrToDouble(strData);
			}
			else if(strKey == "J")
			{
				line_type = 'J';
				id = StrToInt(strData);
			}
			else if (strKey == "S" || strKey == "START" || strKey == "start")
			{
				from = StrToInt(strData);
			}
			else if (strKey == "E" || strKey == "END" || strKey == "end")
			{
				dest = StrToInt(strData);
			}
			else if (strKey == "N" || strKey == "NODES" || strKey == "nodes")
			{
				int number_of_nodes = StrToInt(strData);
				
				if (number_of_nodes<=0 || number_of_nodes>50000)
					return "Bad number of node in HTK file : "+IntToStr(number_of_nodes);
						
				if (graph) delete graph;
				if (nodes) delete [] nodes;
				
				graph = new Graphe(number_of_nodes,i_start>=0?i_start:0,i_end>=0?i_end:number_of_nodes-1);
				nodes = new HTKNode[number_of_nodes];
			}
			else if (strKey == "L" || strKey == "LINKS" || strKey == "links")
			{
				// This is the number of edges. We do not use it.
			}
			else
			{
				if (line_type == 'I')
					nodes[id].values[strKey] = score_opposite?-StrToDouble(strData):StrToDouble(strData);
				
				if (line_type == 'J')
					values[strKey] = score_opposite?-StrToDouble(strData):StrToDouble(strData);
			}
		}
		while (!((pos == string::npos) || (strBuffer[pos] == '\n') || (strBuffer[pos] == '\r') || (strBuffer[pos] == 0)));
		
		if (line_type == 'J')
		{
			// we just parsed an edge line so add the edge
			graph->ajouterArc(from,dest,values,XMLData,id);
			graph->score_to_use = (*values.begin()).first;
		}
	}
	
	int n_nodes = graph->getNbSommets();

	//normalize time
	for(int i=0;i<n_nodes;i++)
		nodes[i].time = nodes[i].time / nodes[n_nodes-1].time;
	
	
	// now, what we want to do is to check for scores on nodes.
	// The method used here is to loop over all edges and take
	// the dest node of the edge. 
	// Then check if the dest node has a score and if so add this score to the edge
	
	
	Arc** S = graph->getSommet();
	for (int i=0;i<n_nodes;i++)
		for(Arc* pt=S[i];pt;pt=pt->suivant())
		{
			map<string, double>& score_to_add = nodes[pt->sommet()].values;
			
			map<string,double>::iterator it;
			for( it = score_to_add.begin(); it != score_to_add.end(); it++ ) 
				pt->addScore(it->first, it->second);

		}
	
	
	return string("");
}

void HTKGraph::getScores(std::map<std::string, double>& s)
{
	graph->getSommet()[0]->getScores(s);
}


bool HTKGraph::createGraph(const char* export_filename)
{
	ofstream dot_graph("lunaviz_graph.dot");
	bool ok=true;
	
	int n_nodes = graph->getNbSommets();
	Arc** S = graph->getSommet();
	
	int* shortest_path=NULL;
	int* shortest_path_by_an_edge = NULL;
	int* id_list_shortest_path = NULL;
	int* id_list_shortest_path_by_an_edge = NULL;

	shortest_path_str = "";
	shortest_path_by_an_edge_str = "";
	map<double, string> map_shortest_path_str;
	map<double, string> map_shortest_path_by_an_edge_str;
	
	// compute the shortest path
	shortest_path = new int[n_nodes];
	id_list_shortest_path = new int[n_nodes];
	memset(shortest_path,0,n_nodes*sizeof(int));
	memset(id_list_shortest_path,-1,n_nodes*sizeof(int));
	distance_shortest_path = shortestPath(shortest_path, id_list_shortest_path, n_nodes);
	

	if (display_shortest_path_by_an_edge)
	{
		shortest_path_by_an_edge = new int[n_nodes];
		id_list_shortest_path_by_an_edge = new int[n_nodes];
		memset(shortest_path_by_an_edge,0,n_nodes*sizeof(int));
		memset(id_list_shortest_path_by_an_edge,-1,n_nodes*sizeof(int));
		distance_shortest_path_by_an_edge = shortestPath(shortest_path_by_an_edge, id_list_shortest_path_by_an_edge, n_nodes, true);
	}
		
	// the graph will be displayed from Left to Right
	dot_graph << "digraph G {" << endl;
	dot_graph << "rankdir=LR;" << endl;

	
	// create nodes
	for(int i=0;i<n_nodes;i++)
	{

		// the first and last nodes have special attributes
		if (i==0)
		{
			dot_graph << IntToStr(i) << " [shape=doublecircle, color=darkorange, style=filled, fillcolor=lightgrey];" << endl;
		}
		
		if (i==n_nodes-1)
		{
			dot_graph << IntToStr(i) << " [shape=doublecircle, color=red, style=filled, fillcolor=lightgrey];" << endl;
		}
	
	}
	
	if (display_shortest_path_by_an_edge)
	{
		for(int i=1;i<n_nodes-1;i++)
			if (shortest_path_by_an_edge[i]==0)
				break;
		else
		{
			dot_graph << IntToStr(shortest_path_by_an_edge[i]) << " [shape=doubleoctagon, color=red, style=filled, fillcolor=cyan];" << endl;
		}
	}
	
	if (display_shortest_path)
	{
		for(int i=1;i<n_nodes-1;i++)
			if (shortest_path[i]==0)
				break;
			else
			{
				dot_graph << IntToStr(shortest_path[i]) << " [shape=doubleoctagon, color=blue, style=filled, fillcolor=yellow];" << endl;
			}
	}	
	
	// create edges
	for (int i=0;i<n_nodes;i++)
		for(Arc* pt=S[i];pt;pt=pt->suivant())
		{
			ostringstream edge;
			string edge_param("color=black");

			edge << i << " -> " << pt->sommet();

			int edge_id = pt->getID();
			
			if (display_shortest_path_by_an_edge)
			{
				if (edge_selected.id == edge_id)
				{
					edge_param = "color=red, fontcolor=red";
				}
				
				if (isInPath(id_list_shortest_path_by_an_edge, n_nodes, pt->getID()))
				{
					if (((edge_selected.id != edge_id))&&((i!=edge_selected.from)||(pt->sommet()!=edge_selected.dest)))
					{
						edge_param = "color=cyan, fontcolor=cyan";
					}

					map_shortest_path_by_an_edge_str[nodes[i].time] = getXMLInfo(pt->getData(), false);
				}
				
			}
			
			// the shorstest path is always computed so we have to get the shortest_path_str too
			if (isInPath(id_list_shortest_path, n_nodes, pt->getID()))
			{
				map_shortest_path_str[nodes[i].time] = getXMLInfo(pt->getData(), false);

				if (display_shortest_path)
					edge_param = "color=blue, fontcolor=blue";
			}
				
			
			string info_to_display;
			
			if (display_score)
				info_to_display = DoubleToStr(pt->valeur());
			else
				info_to_display = getXMLInfo(pt->getData(), m_use_path);
			

			dot_graph << edge.str() << " [ " << edge_param << ",label = \"" << info_to_display << "\", comment=\"ID = " << pt->getID() << "\"];"/*, fontname = \"Monospace\" ];*/ << endl;

		}

	map<double,string>::iterator iter;   
	for( iter = map_shortest_path_str.begin(); iter != map_shortest_path_str.end(); iter++ ) 
		shortest_path_str +=  iter->second + " ";
	for( iter = map_shortest_path_by_an_edge_str.begin(); iter != map_shortest_path_by_an_edge_str.end(); iter++ ) 
		shortest_path_by_an_edge_str +=  iter->second + " ";


	dot_graph << "}" << endl;
	
	dot_graph.flush();
	dot_graph.close();
	
	
	const char* filename = export_filename?export_filename:GRAPH_IMAGE_IMG.c_str();
	
	if (export_format=="") export_format=EXPORT_FORMAT;


	string dot_command = "dot -T"+export_format+" -o "+filename+" lunaviz_graph.dot";
	system(dot_command.c_str());

	export_format = EXPORT_FORMAT;
	

	dot_command = "dot -Tdot -o "+GRAPH_IMAGE_DOT+" lunaviz_graph.dot";
	system(dot_command.c_str());

	delete [] shortest_path;
	delete [] id_list_shortest_path;
	delete [] shortest_path_by_an_edge;
	delete [] id_list_shortest_path_by_an_edge;
			
	return ok;
}




// The shortest path is given by Dijkstra in the reverse order
// For example : [ 16 -> 15 -> 10 -> 5 -> 2 -> 1 ]
double HTKGraph::shortestPath(int* path, int* id_list, int n_nodes, bool with_edge)
{
	if (!graph) return -1;
	
	// Compute the shortest path with the Dijkstra algorithm
	double* distance = new double[n_nodes];
	int* pre = new int[n_nodes];
	double dist = 0;
	
	
	if (with_edge) // do we have to pass by a specific edge ?
	{
		const int from1 = graph->getStart(); // depart node
		const int dest1 = edge_selected.from;// first end node
		const int from2 = edge_selected.dest;// second depart node
		const int dest2 = graph->getEnd(); // second end node
			
		int n=0;
		
		// first we compute the shortest path between the end and the midpoint
		graph->Dijkstra(from2,distance,pre);
		for(int j=dest2;j!=from2;j=pre[j])
			path[n++]=j;
				
		dist = distance[dest2]; // get the distance of this shortest path
		
		path[n++]=from2; // add the midpoint
		
		// next we finally add the shortest path between the midpoint et the very first node
		graph->Dijkstra(from1,distance,pre);
		for(int j=dest1;j!=from1;j=pre[j])
			path[n++]=j;
		
		dist += distance[dest1]; // add the distance of this shortest path to the previous one
		
		dist += edge_selected.value; // add the distance of the midedge (the one who was selected by the user)
	}
	else
	{
		const int from = graph->getStart(); // depart node
		const int dest = graph->getEnd(); // end node
		graph->Dijkstra(from,distance,pre);
		int n=0;
		for(int j=dest;j!=from;j=pre[j])
			path[n++]=j;
		
		dist = distance[dest];
	}
	

	// Now fill the id_list array
	// we actually want an array filled with the id of each edge of the shortest path
	// The problem here is to only take one edge between two nodes : the one with the lowest value
	
	int pos=0;
	double best_val=VAL_MAX;
	int best_id=-1;
	Arc** S = graph->getSommet();
	int from = -1; int dest = -1;
	
	for (int i=0;i<n_nodes;i++)
		for(Arc* pt=S[i];pt;pt=pt->suivant()) // for each edge
		{
			for(int j=1;j<n_nodes;j++)
			{
				if ((path[j]==i) && (path[j-1]==pt->sommet())) // if these nodes are in the shortest path
				{
					// very first time we have these two nodes
					if (from==-1)
					{
						from = i;
						dest = pt->sommet();
						best_val = pt->valeur();
						best_id = pt->getID();
					}
					// if this is not the first time we have these two nodes
					else if ((from == i) && (dest == pt->sommet()))
					{
						if (pt->valeur() < best_val)
						{
							best_val = pt->valeur();
							best_id = pt->getID();
						}
					}
					// first time we have these two nodes
					else
					{
						id_list[pos++]=best_id;
						from = i;
						dest = pt->sommet();
						best_val = pt->valeur();
						best_id = pt->getID();
					}
					break;
				}
			}
		}
		
		
	id_list[pos++]=best_id;
	
	delete distance;
	delete pre;
	
	return dist;
}




// Parse the dot file and add found elements to the list
bool HTKGraph::loadGraphElements(std::list<GraphElement>& graph_elements)
{
	ifstream file(GRAPH_IMAGE_DOT.c_str());
	
	if(!file) return false;
	if (!graph) return false;
	
	graph_elements.clear();
	
	// Does a first pass to avoid edges description on two or more (!) lines
	list<string> lines;
	// parse all lines from the dot file
	while(true)
	{
		if(file.eof()) break;
		
		string line;
		getline(file, line);
		while (line[line.size()-1]=='\\')
		{
			string nextLine;
			getline(file, nextLine);
			line+=nextLine;
		}
		lines.push_back(line);
	}

	file.close();
	
	list<string>::iterator it;
	for(it = lines.begin(); it != lines.end(); it++)
	{
		string strBuffer = (*it);
		string::size_type pos=0;
					
		// find the first non-whitespace character
		pos = strBuffer.find_first_not_of(" \t", pos);
	
		// check for empty lines
		if((pos == string::npos) || (strBuffer[pos] == '\n') || (strBuffer[pos] == '\r') || (strBuffer[pos] == 0))
			continue;
	
		// get the key
		string strKey;
		strKey = strBuffer.substr(pos, strBuffer.find_first_of(" =\t\n\r", pos) - pos);
		pos += strKey.size();
		
		// first we want to get the size of the bounding box
		if (strKey == "graph")
		{
			pos=strBuffer.find_first_of("b", pos); // we want the bounding box
			if (pos == string::npos) continue;
			pos=strBuffer.find_first_of(",", pos)+1;
			pos=strBuffer.find_first_of(",", pos)+1;
			string posX = strBuffer.substr(pos, strBuffer.find_first_of(",", pos) - pos);
			pos += posX.size()+1;
			string posY = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
			pos += posY.size()+1;
			
			dot_width = StrToInt(posX);
			dot_height = StrToInt(posY);

			continue;
		}

		if (strKey == "node")
			continue;
		
		// check the next non-whitespace character to see what we have to deal with
		if (strBuffer[strBuffer.find_first_not_of(" \t", pos)] == '[')
		{
			GraphElement node;
			node.x = 0;
			node.y = 0;
			node.width = 0;
			node.height = 0;
			node.node = true;
			node.from=0;//not used
			node.dest=0;//not used
			node.time=0;
			node.number=StrToInt(strKey);
			
			node.time = nodes[node.number].time;
				
			// find the first "
			pos = strBuffer.find_first_of("\"", pos);
			strKey = strBuffer.substr(pos-4, 4); // get the first 4 letters
			pos++;
		
			if (strKey == "pos=")
			{
				string posX = strBuffer.substr(pos, strBuffer.find_first_of(",", pos) - pos);
				pos += posX.size()+1;
				string posY = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
				pos += posY.size()+1;
			
				node.x = StrToInt(posX);
				node.y = StrToInt(posY);
			}
		
			pos = strBuffer.find_first_of("\"", pos);
			strKey = strBuffer.substr(pos-6, 6); // get the first 6 letters
			pos++;
		
			if (strKey == "width=")
			{
				string posX = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
				
				// in some old graphviz versions there was a "," instead of a "."
				if (posX.find_first_of(",", 0) != string::npos)
					posX[posX.find_first_of(",", 0)] = '.';
				
				pos += posX.size()+1;
				node.width = (int)(StrToDouble(posX)*80); // should be 72 to get the exact width but we want it slightly bigger
			}
		
			pos = strBuffer.find_first_of("\"", pos);
			strKey = strBuffer.substr(pos-7, 7); // get the first 7 letters
			pos++;
		
		
			if (strKey == "height=")
			{
				string posY = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
				
				// in some old graphviz versions there was a "," instead of a "."
				if (posY.find_first_of(",", 0) != string::npos)
					posY[posY.find_first_of(",", 0)] = '.';
				
				pos += posY.size()+1;
				node.height = (int)(StrToDouble(posY)*80); // should be 72 to get the exact height but we want it slightly bigger
			}
		
		
			graph_elements.push_back(node);
		}
		else if (strBuffer[strBuffer.find_first_not_of(" \t", pos)] == '-')
		{
			GraphElement edge;
			edge.x = 0;
			edge.y = 0;
			edge.width = 0;
			edge.height = 0;
			edge.node = false;
			edge.from=StrToInt(strKey);
			edge.dest=0;
			edge.number=0;//not used
			
			pos = strBuffer.find_first_not_of(" \t", pos); // do not care about the whitespace
			strKey = strBuffer.substr(pos, strBuffer.find_first_of(" \t", pos) - pos);
			if (strKey != "->") continue; // shouldn't happen !
			pos += strKey.size();
			pos = strBuffer.find_first_not_of(" \t", pos); // do not care about the whitespace
			strKey = strBuffer.substr(pos, strBuffer.find_first_of(" \t", pos) - pos);
			pos += strKey.size();
			edge.dest=StrToInt(strKey);
			
			
			while (strKey != "label")
			{
				pos = strBuffer.find_first_of("=", pos);
				if (pos == string::npos) // shouldn't happen
				{
					cout << "comment not found in dot file" << endl;
					continue;
				}
				strKey = strBuffer.substr(pos-5, 5); // get the first 5 letters
				pos+=2;
			}
			
			if (strKey == "label")
			{
				// the label may begin with a " so we have to manage this case
				if (strBuffer[pos-1]=='"')
					edge.label = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
				else
					edge.label = strBuffer.substr(pos, strBuffer.find_first_of(" ", pos) - pos);
			}
				
			
			pos+=edge.label.size()+1;

			
			while (strKey != "comment=")
			{
				// find the first "
				pos = strBuffer.find_first_of("\"", pos);
				if (pos == string::npos) // shouldn't happen
				{
					cout << "comment not found in dot file" << endl;
					continue;
				}
				strKey = strBuffer.substr(pos-8, 8); // get the first 8 letters
				pos++;
			}
			
			pos = strBuffer.find_first_of("0123456789", pos);
			string id = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
			edge.id = StrToInt(id);
			
			while (strKey != "lp=")
			{
				// find the first "
				pos = strBuffer.find_first_of("\"", pos);
				if (pos == string::npos) // shouldn't happen
				{
					cout << "lp not found in dot file" << endl;
					continue;
				}
				strKey = strBuffer.substr(pos-3, 3); // get the first 3 letters
				pos++;
			}
			

			string posX = strBuffer.substr(pos, strBuffer.find_first_of(",", pos) - pos);
			pos += posX.size()+1;
			string posY = strBuffer.substr(pos, strBuffer.find_first_of("\"", pos) - pos);
			pos += posY.size()+1;
		
			edge.x = StrToInt(posX);
			edge.y = StrToInt(posY);
			
			
			// Finally let's get some data from the internal graph representation
			Arc** S = graph->getSommet();
			int n_nodes = graph->getNbSommets();
			bool found = false;
			
			for (int i=0;i<n_nodes;i++)
			{
				for(Arc* pt=S[i];pt;pt=pt->suivant())
				{
					if (edge.id == pt->getID())
					{
						edge.XMLData = pt->getData();
						edge.value = pt->valeur();
						found=true; // double break doesn't exist ...
						break;
					}
				}
				if (found) break;
			}
			
			graph_elements.push_back(edge);
		}
						
	}
		
	return true;
}

  
