#include "mainWindow.h"
#include "chooseValueDialog.h"
#include "aboutDialog.h"
#include "graph.h"
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>

/*----------------------------------------------------------------------------------------------------------
|	 --- TODO-LIST ---
|	TODO: Handle deeper XML nodes
|	TODO: Increase zoom speed
|	TODO: Zoom should be centered on mouse coordinates
|
*--------------------------------------------------------------------------------------------------------- */

const int LABEL_WIDTH = 12;
const int LABEL_HEIGHT = 20;
const std::string DEFAULT_TITLE   = "Lunaviz";
const std::string LUNAVIZ_VERSION = "Lunaviz v1.2 (10/2008)";

using namespace std;


static string DoubleToStr(double f)
{
	ostringstream oss;
	oss << fixed << f;
	return oss.str();
}

static void updateUI()
{
	while( Gtk::Main::events_pending() )
	   Gtk::Main::iteration();
}
		
		
MainWindow::MainWindow(const string& filename):
		open_buttons(Gtk::Stock::OPEN),
		file_to_load(filename)
{  
	if (!Glib::thread_supported()) 
		Glib::thread_init();
			
	if (audio.initAudio() == false)
		cout << "openAL intialization failed. You will not be able to play sound" << endl;
	
	scale = 1.0f;
	mouse_x = 0;
	mouse_y = 0;
	resetElementSelected();
	display_shortest_path = false;
	display_shortest_path_selected = false;
	file_opened = false;
	is_reloading = false;
	is_CTRL_pressed = false;
	sound_begin = -1;
	sound_end = -1;
	sound_pos_begin = -1;
	sound_pos_end = -1;
	mouse_pressed_over_sound_area = false;
	m_animation_started = false;
	
	set_title(DEFAULT_TITLE);
	set_default_size(795, 550); // fit in a 800 * 600 screen
	set_icon_from_file(IMAGES_FOLDER+"logo_luna.png");
	add(mainVBox);

	//Create actions for menus and toolbars:
	m_refActionGroup = Gtk::ActionGroup::create();

	m_refActionGroup->add(Gtk::Action::create("FileMenu", "File"));
	m_refActionGroup->add(Gtk::Action::create("FileOpen", Gtk::Stock::OPEN, "Open", "Open a HTK file"), sigc::mem_fun(*this, &MainWindow::onMenuItemOpen));
	m_refActionGroup->add(Gtk::Action::create("FileExport", Gtk::Stock::SAVE_AS, "Export to"));
	m_refActionGroup->add(Gtk::Action::create("FileExportPNG", "PNG", "Export the current graph to PNG format"), sigc::bind<-1, std::string>(sigc::mem_fun(*this, &MainWindow::onMenuItemExport), "png"));
	m_refActionGroup->add(Gtk::Action::create("FileExportSVG", "SVG", "Export the current graph to SVG format"), sigc::bind<-1, std::string>(sigc::mem_fun(*this, &MainWindow::onMenuItemExport), "svg"));
	m_refActionGroup->add(Gtk::Action::create("FileExportDIA", "DIA", "Export the current graph to DIA format"), sigc::bind<-1, std::string>(sigc::mem_fun(*this, &MainWindow::onMenuItemExport), "dia"));	
	m_refActionGroup->add(Gtk::Action::create("FileExportPostscript", "Postscript", "Export the current graph to Postscript format"), sigc::bind<-1, std::string>(sigc::mem_fun(*this, &MainWindow::onMenuItemExport), "ps"));	
	m_refActionGroup->add(Gtk::Action::create("FileExportPostscript2", "Postscript for PDF", "Export the current graph to Postscript for PDF format"), sigc::bind<-1, std::string>(sigc::mem_fun(*this, &MainWindow::onMenuItemExport), "ps2"));
	m_refActionGroup->add(Gtk::Action::create("FileClose", Gtk::Stock::CLOSE, "Close", "Close the file"), sigc::mem_fun(*this, &MainWindow::onMenuItemClose));
	m_refActionGroup->add(Gtk::Action::create("FileQuit", Gtk::Stock::QUIT, "Quit", "Quit the application"), sigc::mem_fun(*this, &MainWindow::onMenuItemQuit));
	m_refActionGroup->add(Gtk::Action::create("DisplayMenu", "Display"));
	m_refActionGroup->add(Gtk::Action::create("DisplayZoomIn", Gtk::Stock::ZOOM_IN, "Zoom in", "Increase zoom"), sigc::mem_fun(*this, &MainWindow::onMenuItemZoomIn));
	m_refActionGroup->add(Gtk::Action::create("DisplayZoomOut", Gtk::Stock::ZOOM_OUT, "Zoom out", "Decrease zoom"), sigc::mem_fun(*this, &MainWindow::onMenuItemZoomOut));
	m_refActionGroup->add(Gtk::Action::create("DisplayZoomFit", Gtk::Stock::ZOOM_FIT, "Zoom fit", "The graph will fit in the window"), sigc::mem_fun(*this, &MainWindow::onMenuItemZoomNormal));
	m_refActionGroup->add(Gtk::Action::create("DisplayShortestPath", Gtk::Stock::GO_FORWARD, "Shortest path", "Display the shortest path on the graph"), sigc::mem_fun(*this, &MainWindow::onMenuItemDisplayShortestPath));
	m_refActionGroup->add(Gtk::Action::create("DisplayShortestPathSelected", Gtk::Stock::GOTO_LAST, "S.P. with edge", "Display the shortest path using the selected edge"), sigc::mem_fun(*this, &MainWindow::onMenuItemDisplayShortestPathSelected));
	m_refActionGroup->add(Gtk::Action::create("Edges", "Edges"));
	m_refActionGroup->add(Gtk::Action::create("DisplayDefault", "Default display (in/out fields)", "Default display (in/out fields)"), sigc::mem_fun(*this, &MainWindow::onMenuChoicesDisplayDefault));
	m_refActionGroup->add(Gtk::Action::create("DisplaySelectedXML", "Display the selected XML field(s)", "Display the selected XML field(s)"), sigc::mem_fun(*this, &MainWindow::onMenuChoicesDisplayXMLSelected));
	m_refActionGroup->add(Gtk::Action::create("DisplayValue", "Display edge value", "Display edge value"), sigc::mem_fun(*this, &MainWindow::onMenuChoicesDisplayValue));
	m_refActionGroup->add(Gtk::Action::create("Scores", "Scores...", "Change the displayed scores"), sigc::mem_fun(*this, &MainWindow::onScores));
	m_refActionGroup->add(Gtk::Action::create("SoundMenu", "Sound"));
	m_refActionGroup->add(Gtk::Action::create("FileOpenSound", Gtk::Stock::OPEN, "Open", "Open the sound file matching the current grah"), sigc::mem_fun(*this, &MainWindow::onMenuItemOpenSound));
	m_refActionGroup->add(Gtk::Action::create("SoundMenuPlay", Gtk::Stock::MEDIA_PLAY, "Play", "Play the sound"), sigc::mem_fun(*this, &MainWindow::onMenuItemPlay));
	m_refActionGroup->add(Gtk::Action::create("SoundMenuStop", Gtk::Stock::MEDIA_STOP, "Stop", "Stop the sound"), sigc::mem_fun(*this, &MainWindow::onMenuItemStop));
	m_refActionGroup->add(Gtk::Action::create("HelpMenu", "Help"));
	m_refActionGroup->add(Gtk::Action::create("HelpAbout", Gtk::Stock::ABOUT, "About", "About "+DEFAULT_TITLE), sigc::mem_fun(*this, &MainWindow::onMenuItemAbout));
	m_refUIManager = Gtk::UIManager::create();
	m_refUIManager->insert_action_group(m_refActionGroup);

	add_accel_group(m_refUIManager->get_accel_group());

	//Layout the actions in a menubar and toolbar:
	Glib::ustring ui_info = 
			"<ui>"
			"  <menubar name='MenuBar'>"
			"    <menu action='FileMenu'>"
			"      <menuitem action='FileOpen'/>"
			"        <menu action='FileExport'>"
			"          <menuitem action='FileExportPNG'/>"
			"          <menuitem action='FileExportSVG'/>"
			"          <menuitem action='FileExportDIA'/>"
			"          <menuitem action='FileExportPostscript'/>"
			"          <menuitem action='FileExportPostscript2'/>"
			"        </menu>"
			"      <menuitem action='FileClose'/>"
			"      <separator/>"
			"      <menuitem action='FileQuit'/>"
			"    </menu>"
			"    <menu action='DisplayMenu'>"
			"      <menuitem action='DisplayZoomIn'/>"
			"      <menuitem action='DisplayZoomOut'/>"
			"      <menuitem action='DisplayZoomFit'/>"
			"      <separator/>"
			"      <menuitem action='DisplayShortestPath'/>"
			"      <menuitem action='DisplayShortestPathSelected'/>"
			"        <menu action='Edges'>"
			"          <menuitem action='DisplayDefault'/>"
			"          <menuitem action='DisplaySelectedXML'/>"
			"          <menuitem action='DisplayValue'/>"
			"        </menu>"
			"      <menuitem action='Scores'/>"
			"    </menu>"
			"    <menu action='SoundMenu'>"
			"      <menuitem action='FileOpenSound'/>"
			"      <menuitem action='SoundMenuPlay'/>"
			"      <menuitem action='SoundMenuStop'/>"
			"    </menu>"
			"    <menu action='HelpMenu'>"
			"      <menuitem action='HelpAbout'/>"
			"    </menu>"
			"  </menubar>"
			"  <toolbar  name='ToolBar'>"
			"    <toolitem action='FileClose'/>"
			"    <separator/>"
			"    <toolitem action='DisplayZoomFit'/>"
			"    <separator/>"
			"    <toolitem action='DisplayShortestPath'/>"
			"    <toolitem action='DisplayShortestPathSelected'/>"
			"    <separator/>"
			"    <toolitem action='SoundMenuPlay'/>"
			"    <toolitem action='SoundMenuStop'/>"
			"  </toolbar>"
			"</ui>";


	
	try
	{
		m_refUIManager->add_ui_from_string(ui_info);
	}
	catch (const Glib::Error& ex)
	{
		// should never happen
		cout << "building menus failed: " <<  ex.what() << endl;
	}

	//Get the menubar and toolbar widgets, and add them to a container widget:
	Gtk::Widget* pMenubar = m_refUIManager->get_widget("/MenuBar");
	if(pMenubar)
		mainVBox.pack_start(*pMenubar, Gtk::PACK_SHRINK);

	Gtk::Toolbar* pToolbar = (Gtk::Toolbar*) m_refUIManager->get_widget("/ToolBar") ;

	if(pToolbar)
	{
		logo1.set(IMAGES_FOLDER+"logo_luna.png");
		logo2.set(IMAGES_FOLDER+"IST-europe.png");
		
		item_logo_1.add(logo1);
		item_logo_2.add(logo2);
		
		pToolbar->insert(item_logo_2, -1);
		pToolbar->insert(item_logo_1, -1);
		
		open_buttons.set_label("Open");
		open_buttons.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onMenuItemOpen));
		pToolbar->insert(open_buttons, 0);
		
		mainVBox.pack_start(*pToolbar, Gtk::PACK_SHRINK);
	}
	
	
	mainVBox.pack_start(hpaned, Gtk::PACK_EXPAND_WIDGET);
	mainVBox.pack_start(status_bar,Gtk::PACK_SHRINK);
	status_bar.push("No file opened");
	
	hpaned.pack1(scroll_window_treeview, Gtk::FILL);
	hpaned.pack2(vpaned, true, false);
	hpaned.set_position(130);
	
	vpaned.pack1(scroll_window_sound, Gtk::FILL);
	vpaned.pack2(vpaned_drawing_area, true, false);
	vpaned.set_position(50);

	vpaned_drawing_area.pack1(m_scrolled_window_text_view, Gtk::SHRINK);
	vpaned_drawing_area.pack2(main_scrolled_window, true, false);
	vpaned_drawing_area.set_position(30);
	
	m_scrolled_window_text_view.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	scroll_window_sound.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	main_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	scroll_window_treeview.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	
	m_scrolled_window_text_view.add(m_text_view);
	scroll_window_sound.add(drawing_area_sound);
	main_scrolled_window.add(main_drawing_area);
	scroll_window_treeview.add(edge_tree_view);

	m_ref_text_buffer_shortest_path = Gtk::TextBuffer::create();
	m_text_view.set_buffer(m_ref_text_buffer_shortest_path);
	m_text_view.set_editable(false);
	m_text_view.set_justification(Gtk::JUSTIFY_CENTER);
	
	main_drawing_area.set_events(Gdk::POINTER_MOTION_MASK | Gdk::KEY_PRESS_MASK | Gdk::KEY_RELEASE_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::SCROLL_MASK);
	drawing_area_sound.set_events(Gdk::POINTER_MOTION_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK);
			
	drawing_area_sound.signal_expose_event().connect(sigc::mem_fun(*this, &MainWindow::onDrawingAreaSoundExposeEvent));
	drawing_area_sound.signal_motion_notify_event().connect(sigc::mem_fun(*this, &MainWindow::onDrawingAreaSoundMotionEvent));
	drawing_area_sound.signal_button_press_event().connect(sigc::mem_fun(*this, &MainWindow::onDrawingAreaSoundPressEvent));
	drawing_area_sound.signal_button_release_event().connect(sigc::mem_fun(*this, &MainWindow::onDrawingAreaSoundReleaseEvent));
	main_drawing_area.signal_expose_event().connect(sigc::mem_fun(*this, &MainWindow::onMainDrawingAreaExposeEvent));
	main_drawing_area.signal_scroll_event().connect(sigc::mem_fun(*this, &MainWindow::onMainScrollEvent));
	main_drawing_area.signal_motion_notify_event().connect(sigc::mem_fun(*this, &MainWindow::onMainDrawingAreaMotionEvent));
	main_drawing_area.signal_button_press_event().connect(sigc::mem_fun(*this, &MainWindow::onMainDrawingAreaPressEvent));
	signal_key_press_event().connect(sigc::mem_fun(*this, &MainWindow::onKeyPressOrReleaseEvent));
	signal_key_release_event().connect(sigc::mem_fun(*this, &MainWindow::onKeyPressOrReleaseEvent));
	signal_show().connect(sigc::mem_fun(*this, &MainWindow::onShow));
	
	refTreeSelection = edge_tree_view.get_selection();
	refTreeSelection->set_mode(Gtk::SELECTION_MULTIPLE);

	show_all_children();	
}

MainWindow::~MainWindow()
{
	
}

void MainWindow::onShow()
{
	if (file_to_load != "")
	{
		onMenuItemOpenFileName(PreOpenFile(file_to_load));
		file_to_load = "";
	}
}


void MainWindow::onMenuItemOpen()
{
	Gtk::FileChooserDialog dialog("Please choose a HTK file",Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.set_transient_for(*this);

	//Add response buttons the the dialog:
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	//Add filters, so that only certain file types can be selected:

	Gtk::FileFilter filter_cpp;
	filter_cpp.set_name("HTK files");
	filter_cpp.add_pattern("*.htk");
	filter_cpp.add_pattern("*.lat");
	dialog.add_filter(filter_cpp);

	Gtk::FileFilter filter_any;
	filter_any.set_name("Any files");
	filter_any.add_pattern("*");
	dialog.add_filter(filter_any);

	//Show the dialog and wait for a user response:
	int result = dialog.run();
	dialog.hide();

	//Handle the response:
	switch(result)
	{
		case(Gtk::RESPONSE_OK):
		{			
			onMenuItemOpenFileName(PreOpenFile(dialog.get_filename()));

			break;
		}
		case(Gtk::RESPONSE_CANCEL):
		{
			break;
		}
		default:
		{
			break;
		}
	}
}



string MainWindow::PreOpenFile(const string& filename)
{
	string graph_filename(filename);
	string graph_base_filename(Glib::path_get_basename(graph_filename));
	graph_path = Glib::path_get_dirname(graph_filename);

	open_menu.items().clear();
	//open_buttons.[un]set_menu();

	// add choices in the open submenu			
	Glib::Dir dir(graph_path);
	std::vector<std::string> entries(dir.begin(), dir.end());

	for(unsigned int i=0;i<entries.size();i++) // loop over the file in the directory
	{
		// get the beginning of the filename (til the first point ("."))
		string entrie_filename(entries[i].substr(0, entries[i].find_first_of(".",0)));
		string graph_base_filename_t = graph_base_filename.substr(0, graph_base_filename.find_first_of(".",0));

		// if we found a filename in the directory which has the same basename (til the first point ("."))
		if (entrie_filename == graph_base_filename_t)
		{
			if (entries[i].substr(entries[i].size()-3,3) == "wav") continue; // do not add wav files
			open_menu.items().push_back(Gtk::Menu_Helpers::MenuElem(entries[i], sigc::bind<-1, std::string>(sigc::mem_fun(*this, &MainWindow::onMenuItemOpenFileName), entries[i])));
		}
	}
	open_buttons.set_menu(open_menu);

	return graph_base_filename;
}



void MainWindow::onMenuItemOpenFileName(const std::string& item)
{
	// close the previous opened file if any
	onMenuItemClose();
	
	string graph_filename(graph_path + "/" +item);
	
	// load the selected HTK file
	string error = htk_graph.parseHTK(graph_filename);
	if (error!="")
	{
		messageBoxError(error);
		return;
	}
			
	file_opened = true;
	onMenuChoicesDisplayDefault();
	onMenuItemZoomNormal();
	status_bar.push(string("graph : ")+graph_filename);
	set_title(DEFAULT_TITLE+" - "+graph_filename);
	
		
	// try to load the matching sound file			
	string::size_type pos=graph_filename.find_first_of(".",0);
	if (pos!=string::npos)
		sound_filename = graph_filename.substr(0, pos) + ".wav";
			
	ifstream sound_file(sound_filename.c_str());
	if (!sound_file)
		is_sound_loaded = false;
	else
	{
		sound_file.close();
		is_sound_loaded = audio.loadSound(sound_filename);
		if (!is_sound_loaded)
			messageBoxError("Unable to load "+sound_filename);
	}
	
	
}

void MainWindow::onMenuItemClose()
{
	if (!file_opened)
		return;
	
	status_bar.pop();
	set_title(DEFAULT_TITLE);
			
	scale = 1.0f;
	mouse_x = 0;
	mouse_y = 0;
	resetElementSelected();
	display_shortest_path = false;
	display_shortest_path_selected = false;
	file_opened = false;
	m_animation_started = false;
	
	GraphElement g;
	htk_graph.displayShortestPathByAnEdge(false, g);
	htk_graph.displayShortestPath(false);
	
	graph_elements.clear();

	
	// clear the tree view
	edge_tree_view.remove_all_columns();
	m_refTreeModel.clear();
	
	main_drawing_area.set_size_request(-1, -1);
	
	graph_image.clear();
	graph_image_original.clear();
	
	audio.unloadSound();
	is_sound_loaded = false;

	m_ref_text_buffer_shortest_path->set_text("");
	
	Gdk::Rectangle r(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
	drawing_area_sound.get_window()->invalidate_rect(r, true);
	
	Gdk::Rectangle r2(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
	main_drawing_area.get_window()->invalidate_rect(r2, true);
}

void MainWindow::onMenuItemAbout()
{
	AboutDialog dialog(LUNAVIZ_VERSION, *this, true);
	dialog.run();
}

void MainWindow::onMenuItemQuit()
{
	Gtk::Main::quit();
}



static void curveRectangle(Cairo::RefPtr<Cairo::Context>& cr, 
							double x0, 
							double y0, 
							double rect_width, 
							double rect_height, 
							double radius, 
							double* rect_color, 
							double* border_color)
{
	double x1,y1;
	
	x1=x0+rect_width;
	y1=y0+rect_height;
	if (!rect_width || !rect_height)
		return;
	if (rect_width/2<radius) {
		if (rect_height/2<radius) {
			cr->move_to  (x0, (y0 + y1)/2);
			cr->curve_to (x0 ,y0, x0, y0, (x0 + x1)/2, y0);
			cr->curve_to (x1, y0, x1, y0, x1, (y0 + y1)/2);
			cr->curve_to (x1, y1, x1, y1, (x1 + x0)/2, y1);
			cr->curve_to (x0, y1, x0, y1, x0, (y0 + y1)/2);
		} else {
			cr->move_to  (x0, y0 + radius);
			cr->curve_to (x0 ,y0, x0, y0, (x0 + x1)/2, y0);
			cr->curve_to (x1, y0, x1, y0, x1, y0 + radius);
			cr->line_to (x1 , y1 - radius);
			cr->curve_to (x1, y1, x1, y1, (x1 + x0)/2, y1);
			cr->curve_to (x0, y1, x0, y1, x0, y1- radius);
		}
	} else {
		if (rect_height/2<radius) {
			cr->move_to  (x0, (y0 + y1)/2);
			cr->curve_to (x0 , y0, x0 , y0, x0 + radius, y0);
			cr->line_to (x1 - radius, y0);
			cr->curve_to (x1, y0, x1, y0, x1, (y0 + y1)/2);
			cr->curve_to (x1, y1, x1, y1, x1 - radius, y1);
			cr->line_to (x0 + radius, y1);
			cr->curve_to (x0, y1, x0, y1, x0, (y0 + y1)/2);
		} else {
			cr->move_to  (x0, y0 + radius);
			cr->curve_to (x0 , y0, x0 , y0, x0 + radius, y0);
			cr->line_to (x1 - radius, y0);
			cr->curve_to (x1, y0, x1, y0, x1, y0 + radius);
			cr->line_to (x1 , y1 - radius);
			cr->curve_to (x1, y1, x1, y1, x1 - radius, y1);
			cr->line_to (x0 + radius, y1);
			cr->curve_to (x0, y1, x0, y1, x0, y1- radius);
		}
	}
	cr->close_path();
	
	cr->set_source_rgba (rect_color[0],rect_color[1],rect_color[2], rect_color[3]);
	cr->fill_preserve();
	cr->set_source_rgba (border_color[0], border_color[1], border_color[2], border_color[3]);
	cr->stroke();
  
}



bool MainWindow::onDrawingAreaSoundExposeEvent(GdkEventExpose* e)
{
	Cairo::RefPtr<Cairo::Context> cr = drawing_area_sound.get_window()->create_cairo_context();

	cr->rectangle(0,0,drawing_area_sound.get_width(),drawing_area_sound.get_height());
	cr->set_source_rgb (1.0,1.0,1.0);
	cr->fill();

	if (!is_sound_loaded) return true;

	short* stream;
	int nb_channel;
	int nb_frames;
	int posY;
	double coeff_y;

	nb_frames = audio.getSoundStream(stream, nb_channel);
	coeff_y = (double)(drawing_area_sound.get_height()-10)/audio.getMaxValue();
	posY = drawing_area_sound.get_height()/2;

	if (!stream) return true;

	nb_frames/=nb_channel;

	int frames_per_pixel = nb_frames/drawing_area_sound.get_width();
	if (frames_per_pixel <= 0)	frames_per_pixel = 1;

	int i=0;
	int num_line=0;
	
	cr->move_to(0,posY);
	while(i<nb_frames)
	{
		double moy_positive=0;
		double moy_negative=0;
		int j;
		for(j=0;j<frames_per_pixel;j++)
		{
			if (i+frames_per_pixel < nb_frames)
			{
				if (stream[i+j]>0)
					moy_positive+=stream[i+j];
				else
					moy_negative+=stream[i+j];
			}
		}
		
		moy_positive/=(double)j;
		moy_negative/=(double)j;
		i+=j;
		
		cr->line_to(num_line, posY+moy_positive*coeff_y);
		cr->line_to(num_line, posY+moy_negative*coeff_y);
		num_line++;
	}
	cr->line_to(num_line, posY);
	
	cr->close_path();
	cr->set_source_rgb (0.0, 0.0, 1.0);
	cr->stroke();

	// draw the selected time
	if (!selected_elements.empty())
	{

		
		double rect_color[] = {1.0,0.0,0.0, 0.3};
		double border_color[] = {0.0, 1.0, 0.0, 0.75};
		curveRectangle(cr, sound_begin*drawing_area_sound.get_width(), 0, (sound_end-sound_begin)*drawing_area_sound.get_width(), drawing_area_sound.get_height(), 20, rect_color, border_color);
	}
	else if (sound_pos_begin != sound_pos_end)// finally draw the selected area by the user
	{
		if (sound_pos_begin > sound_pos_end)
		{
			sound_begin = (double)sound_pos_end/drawing_area_sound.get_width();
			sound_end = (double)sound_pos_begin/drawing_area_sound.get_width();
		}
		else
		{
			sound_begin = (double)sound_pos_begin/drawing_area_sound.get_width();
			sound_end = (double)sound_pos_end/drawing_area_sound.get_width();
		}
		
		double rect_color[] = {1.0,0.0,0.0, 0.3};
		double border_color[] = {0.0, 1.0, 0.0, 0.75};
		curveRectangle(cr, sound_begin*drawing_area_sound.get_width(), 0, (sound_end-sound_begin)*drawing_area_sound.get_width(), drawing_area_sound.get_height(), 20, rect_color, border_color);		
	}

	// if we are playing something draw a line to show where we are
	if (m_animation_started)
	{
		if (selected_elements.empty())
			sound_begin=0;

		int current_frame = (int) ((audio.getPosition(m_animation_elapsed_time)+sound_begin) * nb_frames);
		int posX = current_frame / frames_per_pixel;
		cr->move_to(posX,0);
		cr->line_to(posX, drawing_area_sound.get_height());
		cr->close_path();
		cr->set_source_rgb (1.0, 0.0, 0.0);
		cr->stroke();
	}
	
	return true;
}


bool MainWindow::onDrawingAreaSoundMotionEvent(GdkEventMotion* event)
{
	if (!is_sound_loaded) return true;
	if (!mouse_pressed_over_sound_area) return true;
	
	Gdk::ModifierType m;
	int px, py;
	if (event->is_hint)
		drawing_area_sound.get_window()->get_pointer(px, py, m);
	else
	{
		px = (int)event->x;
		py = (int)event->y;
	}
	
	sound_pos_end = px;
	
	Gdk::Rectangle r(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
	drawing_area_sound.get_window()->invalidate_rect(r, true);
	
	return true;
}


bool MainWindow::onDrawingAreaSoundPressEvent(GdkEventButton* event)
{
	if (!is_sound_loaded) return true;
	
	if (event->button == 1)
	{
		resetElementSelected();
		
		mouse_pressed_over_sound_area = true;
	
		sound_pos_begin = (int)event->x;
		sound_pos_end = (int)event->x;
		
		Gdk::Rectangle r(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
		drawing_area_sound.get_window()->invalidate_rect(r, true);
	}
	
	return true;
}

bool MainWindow::onDrawingAreaSoundReleaseEvent(GdkEventButton* event)
{
	if (!is_sound_loaded) return true;
	
	if (event->button == 1)
	{
		mouse_pressed_over_sound_area = false;
		sound_pos_end = (int)event->x;
			
		// select the edges which are in the selected sound area
		if (sound_pos_end != sound_pos_begin)
		{
			if (sound_pos_begin > sound_pos_end)
			{
				sound_begin = (double)sound_pos_end/drawing_area_sound.get_width();
				sound_end = (double)sound_pos_begin/drawing_area_sound.get_width();
			}
			else
			{
				sound_begin = (double)sound_pos_begin/drawing_area_sound.get_width();
				sound_end = (double)sound_pos_end/drawing_area_sound.get_width();
			}
			
			updateNodesWithSound();

			// we want to redraw the main drawing area
			Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
			main_drawing_area.get_window()->invalidate_rect(r, true);
		}
		
		Gdk::Rectangle r(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
		drawing_area_sound.get_window()->invalidate_rect(r, true);	
		
	}
		
	return true;
}


void MainWindow::updateNodesWithSound()
{
	resetElementSelected();
	
	std::list<GraphElement>::iterator it;
	for(it = graph_elements.begin(); it != graph_elements.end(); it++)
	{
		if ((*it).node) continue; // we want edges only
			
		double node1t = -2;
		double node2t = -2;
		std::list<GraphElement>::iterator itn;
		for(itn = graph_elements.begin(); itn != graph_elements.end(); itn++) // for each node
		{
			if ((*itn).node==false) continue; // we want nodes only

			if ((*itn).number == (*it).from)
			{
				node1t=(*itn).time;
			}

			if ((*itn).number == (*it).dest)
			{
				node2t=(*itn).time;
			}

			if (node1t>-1 && node2t>=-1)
			if ((node1t>=sound_begin) && (node2t<=sound_end))
			{
					// check to see if the element is not already in the list
					bool add = true;
					std::list<GraphElement>::iterator its;
					for(its = selected_elements.begin(); its != selected_elements.end(); its++)
					{
						if ((*its).id == (*it).id)
						{
							add = false;
							break;
						}								
					}
					if (add)
						selected_elements.push_back(*it);
			}
		}
	}
}


void MainWindow::updateSoundWithNodes()
{
	std::list<GraphElement>::iterator its;
	for(its = selected_elements.begin(); its != selected_elements.end(); its++) // for each selected edge
	{
		double from=0, dest=0;
		std::list<GraphElement>::iterator it;
		for(it = graph_elements.begin(); it != graph_elements.end(); it++) // for each node
		{
			if ((*it).node==false) continue; // we want nodes only
			if ((*it).number == (*its).from)
				from = (*it).time;

			if ((*it).number == (*its).dest)
				dest = (*it).time;
		}
			
		if (from == dest)
		{
			from--;
			dest++;
		}
		else if (from > dest)
		{
			double tmp = from;
			from = dest;
			dest = tmp;
		}
			
		if (from < sound_begin) sound_begin = from;
		if (dest > sound_end) sound_end = dest;
	}
}




bool MainWindow::onMainDrawingAreaExposeEvent(GdkEventExpose* e)
{		
	Cairo::RefPtr<Cairo::Context> cr = main_drawing_area.get_window()->create_cairo_context();
	
	cr->rectangle(0,0,main_drawing_area.get_width(),main_drawing_area.get_height());
	cr->set_source_rgb (1.0,1.0,1.0);
	cr->fill();
	
	if (is_reloading) return true;
	if (!graph_image) return true;
	
	int posX = (int)(main_scrolled_window.get_hadjustment()->get_value());
	int posY = (int)(main_scrolled_window.get_vadjustment()->get_value());
	int width  = main_scrolled_window.get_width();
	int height = main_scrolled_window.get_height();
	
	if (posX+width>graph_image->get_width()) width=graph_image->get_width()-posX;
	if (posY+height>graph_image->get_height()) height=graph_image->get_height()-posY;
	
	// If there is no scroll bar, we want to draw the image to the center of the drawing area
	int destX = (posX==0)?main_drawing_area.get_width()/2-graph_image->get_width()/2:posX;
	int destY = (posY==0)?main_drawing_area.get_height()/2-graph_image->get_height()/2:posY;

	main_drawing_area.get_window()->draw_pixbuf( main_drawing_area.get_style()->get_black_gc(),
								graph_image,
								posX, // X in source image 
								posY, // Y in source image
								destX,// X in dest image
								destY,// Y in dest image
								width, height, // width and weight
								Gdk::RGB_DITHER_NONE, 0, 0);
	
	

	
	float coeff_x = (float)graph_image_original->get_width() / htk_graph.dot_width;
	float coeff_y = (float)graph_image_original->get_height() / htk_graph.dot_height;
	

	// First let's highlight the selected elements
	std::list<GraphElement>::iterator its;
	for(its = selected_elements.begin(); its != selected_elements.end(); its++)
	{
		GraphElement& element_selected = (*its);
		if (element_selected.x!=-1)
		{
			GraphElement n;
				
			const int N_LABEL_WIDTH = LABEL_WIDTH * element_selected.label.size();
				
			n.x = (int)(((element_selected.x*coeff_x)-N_LABEL_WIDTH/2.0f) * scale + destX - posX);
			n.y = (int)(((graph_image_original->get_height()-(element_selected.y*coeff_y))-LABEL_HEIGHT/3.0f) * scale + destY - posY);
			n.width = (int)(N_LABEL_WIDTH * scale);
			n.height = (int)(LABEL_HEIGHT * scale);
			n.from = element_selected.from;
			n.dest = element_selected.dest;
			n.id = element_selected.id;
			
			double rect_color[] = {1.0, 0.0, 0.0, 0.15};
			double border_color[] = {1.0, 0.0, 0.0, 0.5};
			curveRectangle(cr, n.x, n.y, n.width, n.height, n.height, rect_color, border_color);
			
			std::list<GraphElement>::iterator it2;
			for(it2 = graph_elements.begin(); it2 != graph_elements.end(); it2++)
			{
				if ((*it2).node==false) continue; // we want nodes only
					
				if (((*it2).number == n.from) || ((*it2).number == n.dest))
				{
					GraphElement n2;
					n2.x = (int)((((*it2).x*coeff_x)-(*it2).width*coeff_x/2.0f) * scale + destX - posX);
					n2.y = (int)(((graph_image_original->get_height()-((*it2).y*coeff_y))-(*it2).height*coeff_y/2.0f) * scale + destY - posY);
					n2.width = (int)((*it2).width*coeff_y * scale);
					n2.height = (int)((*it2).height*coeff_y * scale);
							
					double rect_color[] = {0.0,1.0,0.0, 0.4};
					double border_color[] = {0.0, 0.0, 1.0, 0.7};
					curveRectangle(cr, n2.x, n2.y, n2.width, n2.height, n2.height, rect_color, border_color);
							
				}
			}
			
			
		}
	}

	
	// Now let's highlight the edge which is under the mouse
	std::list<GraphElement>::iterator it;
	for(it = graph_elements.begin(); it != graph_elements.end(); it++)
	{
		if ((*it).node) continue; // we want edges only
		
		// if we are over a selected edge, we don't want to highlight it
		//if ((*it).id == element_selected.id)
			//continue;
				

		GraphElement n;
		
		const int N_LABEL_WIDTH = LABEL_WIDTH * (*it).label.size();
		
		n.x = (int)((((*it).x*coeff_x)-N_LABEL_WIDTH/2.0f) * scale + destX - posX);
		n.y = (int)(((graph_image_original->get_height()-((*it).y*coeff_y))-LABEL_HEIGHT/3.0f) * scale + destY - posY);
		n.width = (int)(N_LABEL_WIDTH * scale);
		n.height = (int)(LABEL_HEIGHT * scale);
		n.from = (*it).from;
		n.dest = (*it).dest;
		n.id = (*it).id;
				
		if ((mouse_x > n.x) && (mouse_x < n.x+n.width) && (mouse_y > n.y) && (mouse_y < n.y+n.height))
		{
			double rect_color[] = {0.0, 0.0, 1.0, 0.15};
			double border_color[] = {0.0, 0.0, 1.0, 0.5};
			curveRectangle(cr, n.x, n.y, n.width, n.height, n.height, rect_color, border_color);

			std::list<GraphElement>::iterator it2;
			for(it2 = graph_elements.begin(); it2 != graph_elements.end(); it2++)
			{
				if ((*it2).node==false) continue; // we want nodes only
				
				if (((*it2).number == n.from) || ((*it2).number == n.dest))
				{
					GraphElement n2;
					n2.x = (int)((((*it2).x*coeff_x)-(*it2).width*coeff_x/2.0f) * scale + destX - posX);
					n2.y = (int)(((graph_image_original->get_height()-((*it2).y*coeff_y))-(*it2).height*coeff_y/2.0f) * scale + destY - posY);
					n2.width = (int)((*it2).width*coeff_y * scale);
					n2.height = (int)((*it2).height*coeff_y * scale);
						
					double rect_color[] = {1.0,1.0,0.0, 0.4};
					double border_color[] = {1.0, 0.0, 0.0, 0.7};
					curveRectangle(cr, n2.x, n2.y, n2.width, n2.height, n2.height, rect_color, border_color);	
				}	
			}	
		}
	}


	if (m_animation_started)
	{
		double time_pos = audio.getPosition(m_animation_elapsed_time)+sound_begin;

		if (time_pos >= 0.01)
		{
			std::list<GraphElement>::iterator it;
			for(it = graph_elements.begin(); it != graph_elements.end(); it++)
			{
				if ((*it).node) continue; // we want edges only


				std::list<GraphElement>::iterator it2;
				for(it2 = graph_elements.begin(); it2 != graph_elements.end(); it2++) // for each node
				{
					if ((*it2).node==false) continue; // we want nodes only


					if ((*it2).time > time_pos  && (*it2).time < time_pos+0.08 )
					{
						GraphElement n2;
						n2.x = (int)((((*it2).x*coeff_x)-(*it2).width*coeff_x/2.0f) * scale + destX - posX);
						n2.y = (int)(((graph_image_original->get_height()-((*it2).y*coeff_y))-(*it2).height*coeff_y/2.0f) * scale + destY - posY);
						n2.width = (int)((*it2).width*coeff_y * scale);
						n2.height = (int)((*it2).height*coeff_y * scale);

						// since a lot of rectangle may potentially be drawn here, we won't use curveRectangle since it takes
						// a lot of computation time
						//double rect_color[] = {1.0,1.0,0.0, 0.4};
						//double border_color[] = {1.0, 0.0, 0.0, 0.7};
						//curveRectangle(cr, n2.x, n2.y, n2.width, n2.height, n2.height, rect_color, border_color);	
						cr->rectangle (n2.x, n2.y, n2.width, n2.height);
						cr->set_source_rgb(0.0,1.0,0.0);
						cr->fill();
					}

				}
			}
		}

	}

	return true;
}

bool MainWindow::onKeyPressOrReleaseEvent(GdkEventKey* event)
{
	if (event->type == GDK_KEY_RELEASE)
		is_CTRL_pressed = (event->keyval != GDK_Control_L);
	else
		is_CTRL_pressed = (event->keyval == GDK_Control_L);

	return true;
}


bool MainWindow::onMainScrollEvent(GdkEventScroll* event)
{
	//TODO: look at the mouse position to zoom in the right place
	//Actually we have to change the scroll bar position to do this
	if (event->direction == GDK_SCROLL_UP)
	{
		get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
		updateUI();

		onMenuItemZoomIn();

		get_window()->set_cursor(Gdk::Cursor(Gdk::LEFT_PTR));
		updateUI();
	}
	else if (event->direction == GDK_SCROLL_DOWN)
	{
		get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
		updateUI();

		onMenuItemZoomOut();

		get_window()->set_cursor(Gdk::Cursor(Gdk::LEFT_PTR));
		updateUI();
	}

	return true;
}


bool MainWindow::onMainDrawingAreaMotionEvent(GdkEventMotion* event)
{
	if (is_reloading) return true;
	if (!graph_image) return true;

	static bool the_mouse_is_over_an_edge = false;

	Gdk::ModifierType m;
	if (event->is_hint)
		main_drawing_area.get_window()->get_pointer(mouse_x, mouse_y, m);
	else
	{
		mouse_x = (int) event->x;
		mouse_y = (int) event->y;
	}

	
	int posX = (int)(main_scrolled_window.get_hadjustment()->get_value());
	int posY = (int)(main_scrolled_window.get_vadjustment()->get_value());
	int width  = main_scrolled_window.get_width();
	int height = main_scrolled_window.get_height();
	
	if (posX+width>graph_image->get_width()) width=graph_image->get_width()-posX;
	if (posY+height>graph_image->get_height()) height=graph_image->get_height()-posY;
	
	// If there is no scroll bar, we want to draw the image to the center of the drawing area
	int destX = (posX==0)?main_drawing_area.get_width()/2-graph_image->get_width()/2:posX;
	int destY = (posY==0)?main_drawing_area.get_height()/2-graph_image->get_height()/2:posY;

	float coeff_x = (float)graph_image_original->get_width() / htk_graph.dot_width;
	float coeff_y = (float)graph_image_original->get_height() / htk_graph.dot_height;
	
	std::list<GraphElement>::iterator it;
	for(it = graph_elements.begin(); it != graph_elements.end(); it++)
	{
		if ((*it).node) continue; // we want edges only

		GraphElement n;
		
		const int N_LABEL_WIDTH = LABEL_WIDTH * (*it).label.size();
		
		n.x = (int)((((*it).x*coeff_x)-N_LABEL_WIDTH/2.0f) * scale + destX - posX);
		n.y = (int)(((graph_image_original->get_height()-((*it).y*coeff_y))-LABEL_HEIGHT/3.0f) * scale + destY - posY);
		n.width = (int)(N_LABEL_WIDTH * scale);
		n.height = (int)(LABEL_HEIGHT * scale);
		n.from = (*it).from;
		n.dest = (*it).dest;
		
		
		if ((event->x > n.x) && (event->x < n.x+n.width) && (event->y > n.y) && (event->y < n.y+n.height))
		{
			Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
			main_drawing_area.get_window()->invalidate_rect(r, true);
			the_mouse_is_over_an_edge = true;
			return true;
		}
	}
	
	// if we were over an edge the previous time we want to clear the screen
	if (the_mouse_is_over_an_edge)
	{
		Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
		main_drawing_area.get_window()->invalidate_rect(r, true);
		the_mouse_is_over_an_edge = false;
	}
	
	return true;
}


bool MainWindow::onMainDrawingAreaPressEvent(GdkEventButton* event)
{
	if (is_reloading) return true;
	if (!graph_image) return true;
	if (!((event->type == GDK_BUTTON_PRESS) && (event->button == 1))) return true;
	
	sound_pos_begin = -1;
	sound_pos_end = -1;
	
	mouse_x = (int) event->x;
	mouse_y = (int) event->y;
	
	int posX = (int)(main_scrolled_window.get_hadjustment()->get_value());
	int posY = (int)(main_scrolled_window.get_vadjustment()->get_value());
	int width  = main_scrolled_window.get_width();
	int height = main_scrolled_window.get_height();
	
	if (posX+width>graph_image->get_width()) width=graph_image->get_width()-posX;
	if (posY+height>graph_image->get_height()) height=graph_image->get_height()-posY;
	
	// If there is no scroll bar, we want to draw the image to the center of the drawing area
	int destX = (posX==0)?main_drawing_area.get_width()/2-graph_image->get_width()/2:posX;
	int destY = (posY==0)?main_drawing_area.get_height()/2-graph_image->get_height()/2:posY;
	
	float coeff_x = (float)graph_image_original->get_width() / htk_graph.dot_width;
	float coeff_y = (float)graph_image_original->get_height() / htk_graph.dot_height;
	
	std::list<GraphElement>::iterator it;
	for(it = graph_elements.begin(); it != graph_elements.end(); it++)
	{
		if ((*it).node) continue; // we want edges only
		
		GraphElement n;
		
		const int N_LABEL_WIDTH = LABEL_WIDTH * (*it).label.size();
		
		n.x = (int)((((*it).x*coeff_x)-N_LABEL_WIDTH/2.0f) * scale + destX - posX);
		n.y = (int)(((graph_image_original->get_height()-((*it).y*coeff_y))-LABEL_HEIGHT/3.0f) * scale + destY - posY);
		n.width = (int)(N_LABEL_WIDTH * scale);
		n.height = (int)(LABEL_HEIGHT * scale);
		
		if ((mouse_x > n.x) && (mouse_x < n.x+n.width) && (mouse_y > n.y) && (mouse_y < n.y+n.height))
		{
			// if the user doesn't hold the CTRL key, then clear the list
			if (!is_CTRL_pressed)
				selected_elements.clear();
			
			// Check the element is not already in the list, if it is, remove it
			bool add_edge = true;
			std::list<GraphElement>::iterator its;
			for(its = selected_elements.begin(); its != selected_elements.end(); its++)
			{
				if ((*its).id == (*it).id)
				{
					add_edge = false;
					selected_elements.erase(its);
					break;
				}
			}
			
			if (add_edge)
				selected_elements.push_back(*it);

			sound_begin = 1;
			sound_end = 0;
		
			updateSoundWithNodes();
					
			// we want to redraw the main drawing area
			Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
			main_drawing_area.get_window()->invalidate_rect(r, true);
			// and the sound drawing area
			Gdk::Rectangle r2(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
			drawing_area_sound.get_window()->invalidate_rect(r2, true);
			// update the tree view to show the XML information of the selected node
			updateTreeView();
			
			return true;
		}
	}
	
	// If the user didn't click on an edge, cancel selection
	resetElementSelected();
	
	// we want to redraw the main drawing area
	Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
	main_drawing_area.get_window()->invalidate_rect(r, true);
	// and the sound drawing area
	Gdk::Rectangle r2(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
	drawing_area_sound.get_window()->invalidate_rect(r2, true);
	
	return true;
}



void MainWindow::createTreeView(void* root_node, Gtk::TreeModel::Row& row)
{
	// Add the element from the root recursively
	xmlNode* a_node = (xmlNode*) root_node;
	xmlNode* cur_node = NULL;

	for(cur_node = a_node; cur_node; cur_node = cur_node->next) 
	{
		if (cur_node->type == XML_ELEMENT_NODE) 
		{
			//printf("node type: Element, name: %s\n", cur_node->name);
			row[m_Columns.col_id] = (const char*)(cur_node->name);
		}
		if (cur_node->properties != NULL)
		{
			xmlAttr* attr;
			for(attr = cur_node->properties; attr; attr = attr->next)
			{
				//printf("ATTRIBUTE NAME: %s\n", attr->name);
				//printf("ATTRIBUTE VAL: %s\n", attr->children->content);
				Gtk::TreeModel::Row new_row = *(m_refTreeModel->append(row.children()));
				new_row[m_Columns.col_id] = (const char*)(attr->name);
				new_row[m_Columns.col_attrib] = (const char*)(attr->children->content);
			}
		}
		
		if (cur_node->children)
		{	
			Gtk::TreeModel::Row childrow = *(m_refTreeModel->append(row.children()));
			createTreeView(cur_node->children, childrow);
		}
	}
}


void MainWindow::updateTreeView()
{
	if (selected_elements.empty()) return;
	GraphElement& element_selected = (*selected_elements.begin());
	
	xmlDocPtr doc; // the resulting document tree

	doc = xmlReadMemory(element_selected.XMLData.c_str(), element_selected.XMLData.size(), "noname.xml", NULL, 0);
	if (doc == NULL) 
	{
		cout << "Failed to parse document" << endl; //TODO: DIALOG BOX ?
		return;
	}
	
	// Get the root element node
	xmlNode* root_element = xmlDocGetRootElement(doc);
	
	// Clean the tree view first
	edge_tree_view.remove_all_columns();
	m_refTreeModel.clear();

	// Create the Tree model:
	m_refTreeModel = Gtk::TreeStore::create(m_Columns);
	//edge_tree_view.set_reorderable(); //All the items to be reordered with drag-and-drop
	Gtk::TreeModel::Row row = *(m_refTreeModel->append());
	
	createTreeView(root_element, row);
	
	xmlFreeDoc(doc);
	xmlCleanupParser();

	//Add the TreeView's view columns:
	edge_tree_view.append_column("Attribute", m_Columns.col_id);
	edge_tree_view.append_column("Value", m_Columns.col_attrib);
	
	edge_tree_view.set_model(m_refTreeModel);
	edge_tree_view.expand_all();
}



void MainWindow::onMenuItemZoomIn()
{
	scale*=5.0f/4.0f;
	updateScale();
	updateZoom();
}


void MainWindow::onMenuItemZoomOut()
{
	scale*=4.0f/5.0f;
	updateScale();
	updateZoom();
}


void MainWindow::onMenuItemZoomNormal()
{
	if (!graph_image_original) return;
	
	scale = (float)(main_scrolled_window.get_width()-30) / graph_image_original->get_width();
	updateScale();
	updateZoom();
}

void MainWindow::updateScale()
{
	if (!graph_image_original) return;
	
	if (scale<0.2f) scale=0.2f;
	if (scale>1.0f) scale=1.0f;
	

	graph_image = graph_image_original->scale_simple(
			(int)(graph_image_original->get_width()*scale), 
			 (int)(graph_image_original->get_height()*scale), 
			  Gdk::INTERP_BILINEAR/*Gdk::INTERP_NEAREST*/);
	

	
}

void MainWindow::updateZoom()
{
	if (!graph_image_original) return;
	
	main_drawing_area.set_size_request(graph_image->get_width(),graph_image->get_height());
}


void MainWindow::onMenuItemDisplayShortestPath()
{
	if (!graph_image_original) return;
	display_shortest_path = ! display_shortest_path;

	htk_graph.displayShortestPath(display_shortest_path);
	
	reloadGraph();

}



void MainWindow::onMenuItemDisplayShortestPathSelected()
{
	if (!graph_image_original) return;
	
	GraphElement element_selected;
	
	if (selected_elements.empty())
	{
		// if we don't have anything selected *and* nothing was selected *before* then
		// we don't have anything to do
		if (false == display_shortest_path_selected)
			return;

		display_shortest_path_selected = false;
	}
	else
	{
		display_shortest_path_selected = true;
		element_selected = (*selected_elements.begin());
	}
	
	htk_graph.displayShortestPathByAnEdge( display_shortest_path_selected, element_selected);
	
	reloadGraph();

}

void MainWindow::reloadGraph()
{
	if (!file_opened) return;
	if (is_reloading) return;

	get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
	updateUI();

	Gdk::Rectangle r2(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
	drawing_area_sound.get_window()->invalidate_rect(r2, true);
	
	is_reloading = true;

	status_bar.push("Creating graph...",1);
	updateUI();
	htk_graph.createGraph();

	status_bar.pop(1);
	status_bar.push("Loading graph elements...",1);
	updateUI();
	htk_graph.loadGraphElements(graph_elements);

	status_bar.pop(1);
	status_bar.push("Loading graph image...",1);
	updateUI();
	graph_image_original = Gdk::Pixbuf::create_from_file(GRAPH_IMAGE_IMG);


	status_bar.pop(1);
	updateScale();
	is_reloading = false;
	updateZoom();

	get_window()->set_cursor(Gdk::Cursor(Gdk::LEFT_PTR));
	
	// update the selected element to get the values of the new graph
	std::list<GraphElement>::iterator it;
	for(it = graph_elements.begin(); it != graph_elements.end(); it++)
	{
		std::list<GraphElement>::iterator its;
		for(its = selected_elements.begin(); its != selected_elements.end(); its++)
		{	
			GraphElement& element_selected = (*its);
			if (element_selected.id == (*it).id)
			{
				element_selected = (*it);
				// update the tree view to show the XML information of the selected node
				updateTreeView();
				break;
			}
		}
	}
	
	// we want to redraw the main drawing area
	Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
	main_drawing_area.get_window()->invalidate_rect(r, true);
	// we want to redraw the sound drawing area
	drawing_area_sound.get_window()->invalidate_rect(r2, true);
	
	if ((!display_shortest_path) && (!display_shortest_path_selected))
		status_bar.pop(1);
	else
	{
		status_bar.pop(1);
		string shortest_path("Shortest Path : ");
		string shortest_path_selected;

		shortest_path += ""+DoubleToStr(htk_graph.getDistance());
	
		if (htk_graph.display_shortest_path_by_an_edge)
			shortest_path_selected = "  ---  Shortest Path with the selected edge : "+DoubleToStr(htk_graph.getDistanceByAnEdge());
	
		status_bar.push(shortest_path + shortest_path_selected,1);
	}

	string shortest_path_text;
	string shortest_path_by_an_edge_text;
	display_shortest_path? shortest_path_text="Shortest path : "+htk_graph.getShortestPathStr()+"\n":"";
	display_shortest_path_selected? shortest_path_by_an_edge_text = "Shortest path with the selected edge : "+htk_graph.getShortestPathByAnEdgeStr():"";

	m_ref_text_buffer_shortest_path->set_text(shortest_path_text+shortest_path_by_an_edge_text);
}


void MainWindow::resetElementSelected()
{
	selected_elements.clear();
	
	// clear the tree view too
	edge_tree_view.remove_all_columns();
	m_refTreeModel.clear();
}

void MainWindow::onMenuChoicesDisplayValue()
{
	if (!file_opened) return;

	htk_graph.DisplayScoreOnEdges(true);
	reloadGraph();
}

void MainWindow::onMenuChoicesDisplayDefault()
{
	if (!file_opened) return;

	htk_graph.DisplayScoreOnEdges(false);
	
	XMLAttrPos p1,p2;
	p1.attr_name = "in";
	p1.attr_pos.push_back(1);
	p2.attr_name = "out";
	p2.attr_pos.push_back(1);
	
	vector<XMLAttrPos> xml_attr;
	
	xml_attr.push_back(p1);
	xml_attr.push_back(p2);
	
	htk_graph.DisplayXMLOnEdges(xml_attr);
	
	reloadGraph();
}

void MainWindow::onMenuChoicesDisplayXMLSelected()
{
	if (!file_opened) return;
	
	if (refTreeSelection->count_selected_rows() <= 0)
	{
		messageBoxError("You need to select at least one attribute in the tree view");
		return;
	}
	
	htk_graph.DisplayScoreOnEdges(false);
	vector< vector<int> > xml_path;
	
	vector<Gtk::TreePath> selected_rows(refTreeSelection->get_selected_rows());
	
	for(unsigned int i=0;i<selected_rows.size();i++)
	{
		vector<int>path;
		Gtk::TreePath::iterator it;
		for(it = selected_rows[i].begin(); it != selected_rows[i].end(); it++)
		{
			path.push_back(*it);			
		}
		xml_path.push_back(path);
	}
	
	htk_graph.DisplaySelectedXMLOnEdges(xml_path);
	
	reloadGraph();
	
}


void MainWindow::onScores()
{
	if (!file_opened) return;
	
	ChooseValueDialog d(DEFAULT_TITLE, *this, true, true);
	
	std::map<std::string,double> scores;
	
	htk_graph.getScores(scores);
	
	std::map<std::string,double>::iterator iter;
	for( iter = scores.begin(); iter != scores.end(); iter++ ) 
		d.addChoice(iter->first);
	
	d.setCurrentChoice(Graphe::score_to_use);
	d.setOppositeScore(htk_graph.getOppositeScore());
	d.updateDialog();
	
	int result = d.run();
	
	if (result != Gtk::RESPONSE_OK)
		return;
	
	htk_graph.takeTheOppositeScore(d.getOppositeScore(), graph_elements);
	Graphe::score_to_use = d.getChoice();
	
	reloadGraph();
		
}

void MainWindow::messageBoxError(const std::string& msg)
{
	Gtk::MessageDialog error_dialog(*this, "Error :",false, Gtk::MESSAGE_ERROR,Gtk::BUTTONS_OK);
	error_dialog.set_secondary_text(msg);
	error_dialog.run();
}

void MainWindow::onMenuItemOpenSound()
{
	Gtk::FileChooserDialog dialog("Please choose a sound file",Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.set_transient_for(*this);

	//Add response buttons the the dialog:
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	//Add filters, so that only certain file types can be selected:

	Gtk::FileFilter filter_cpp;
	filter_cpp.set_name("Sound files");
	filter_cpp.add_pattern("*.wav");
	filter_cpp.add_pattern("*.raw");
	dialog.add_filter(filter_cpp);

	Gtk::FileFilter filter_any;
	filter_any.set_name("Any files");
	filter_any.add_pattern("*");
	dialog.add_filter(filter_any);

	//Show the dialog and wait for a user response:
	int result = dialog.run();
	dialog.hide();
	
	if (result == Gtk::RESPONSE_OK)
	{	
		sound_filename = dialog.get_filename();
		is_sound_loaded = audio.loadSound(sound_filename);
		if (!is_sound_loaded)
			messageBoxError("Unable to load "+sound_filename);
		else
		{
			Gdk::Rectangle r(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
			drawing_area_sound.get_window()->invalidate_rect(r, true);
		}
	}
}


void MainWindow::onMenuItemPlay()
{
	// if we are playing a sound we have to wait for it to finish
	if (m_animation_started) return;
	// let's start the animation
	m_animation_started = true;
	m_animation_elapsed_time = 0;
	GTimeVal t;
	g_get_current_time(&t);
	m_animation_start = t.tv_sec*1000 + t.tv_usec/1000; // milliseconds;
	// launch the animation timer
	Glib::signal_timeout().connect(sigc::mem_fun(*this,&MainWindow::onUpdateAnimation),5);

	// set the part of the sound to play
	if (selected_elements.empty())
		audio.setSequenceToPlay(0,1);
	else
		audio.setSequenceToPlay(sound_begin,sound_end);

	// play the sound
	audio.playSound();
}

bool MainWindow::onUpdateAnimation()
{
	if (audio.is_playing())
	{
		GTimeVal t;
		g_get_current_time(&t);
		m_animation_elapsed_time=(t.tv_sec*1000+t.tv_usec/1000)-m_animation_start; // milliseconds delay;

		// we want to redraw the main drawing area
		Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
		main_drawing_area.get_window()->invalidate_rect(r, true);
		// and the sound drawing area
		Gdk::Rectangle r2(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
		drawing_area_sound.get_window()->invalidate_rect(r2, true);

		return true;
	}
	
	// else
	
	// we want to redraw the main drawing area
	Gdk::Rectangle r(0, 0, main_drawing_area.get_width(),main_drawing_area.get_height());
	main_drawing_area.get_window()->invalidate_rect(r, true);
	// and the sound drawing area
	Gdk::Rectangle r2(0, 0, drawing_area_sound.get_width(),drawing_area_sound.get_height());
	drawing_area_sound.get_window()->invalidate_rect(r2, true);
	
	m_animation_started = false;
	return false; // by returning false we stop the timer
}

void MainWindow::onMenuItemStop()
{
	audio.stopSound();
}

void MainWindow::onMenuItemExport(const string& export_format)
{	
	if (!file_opened) 
	{
		messageBoxError("No graph opened");
		return;
	}
	
	Gtk::FileChooserDialog dialog("Please choose a filename",Gtk::FILE_CHOOSER_ACTION_SAVE);
	dialog.set_transient_for(*this);

	//Add response buttons the the dialog:
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
	
	//Show the dialog and wait for a user response:
	if (dialog.run() != Gtk::RESPONSE_OK) return;

	string format(export_format);
	htk_graph.set_export_format(format);
	
	// now we have set the export format to whatever the user have chosen.
	// What we are going to do now is to check that the filename has the
	// appropriate extension. However, there is only one exception here :
	// if the format is ps2 the extension has to be "ps" not "ps2"
	if (format == "ps2") 
		format = "ps";
	
	string filename(dialog.get_filename());
	
	// add the extension if the user didn't.
	if (filename.substr(filename.size()-format.size(),format.size()) != format ) 
		filename=filename+"."+format;
	
	htk_graph.createGraph(filename.c_str());
}








