#ifndef __MAIN_WINDOW__
#define __MAIN_WINDOW__

#include <gtkmm.h>
#include "htkGraph.h"
#include "audioIO.h"


class MainWindow : public Gtk::Window
{
	public:
		MainWindow(const std::string& filename);
		virtual ~MainWindow();
	
	protected:
		// Signal handlers:
		virtual void onMenuItemOpen();
		virtual void onMenuItemOpenFileName(const std::string& item);
		virtual void onMenuItemExport(const std::string& export_format);
		virtual void onMenuItemOpenSound();
		virtual void onMenuItemClose();
		virtual void onMenuItemQuit();
		virtual void onMenuItemAbout();
		virtual void onMenuItemZoomIn();
		virtual void onMenuItemZoomOut();
		virtual void onMenuItemZoomNormal();
		virtual void onMenuItemDisplayShortestPath();
		virtual void onMenuItemDisplayShortestPathSelected();
		virtual void onMenuChoicesDisplayValue();
		virtual void onMenuChoicesDisplayXMLSelected();
		virtual void onMenuChoicesDisplayDefault();
		virtual void onScores();
		virtual void onMenuItemPlay();
		virtual void onMenuItemStop();
		virtual void onShow();
		virtual bool onDrawingAreaSoundMotionEvent(GdkEventMotion* event);
		virtual bool onDrawingAreaSoundPressEvent(GdkEventButton* event);
		virtual bool onDrawingAreaSoundReleaseEvent(GdkEventButton* event);
		virtual bool onDrawingAreaSoundExposeEvent(GdkEventExpose* e);
		virtual bool onMainDrawingAreaExposeEvent(GdkEventExpose* e);
		virtual bool onMainScrollEvent(GdkEventScroll* event);
		virtual bool onMainDrawingAreaMotionEvent(GdkEventMotion* event);
		virtual bool onMainDrawingAreaPressEvent(GdkEventButton* event);
		virtual bool onKeyPressOrReleaseEvent(GdkEventKey* event);
		virtual bool onUpdateAnimation();
		
		
		//Child widgets:
		Gtk::VBox mainVBox;
		Gtk::HPaned hpaned;
		Gtk::VPaned vpaned, vpaned_drawing_area;
		Gtk::ScrolledWindow scroll_window_treeview;
		Gtk::ScrolledWindow scroll_window_sound;
		Gtk::ScrolledWindow main_scrolled_window;
		Gtk::TreeView edgetreeview;
		Gtk::DrawingArea main_drawing_area;
		Gtk::DrawingArea drawing_area_sound;
		Gtk::TreeView edge_tree_view;
		Gtk::Statusbar status_bar;
		Gtk::Image logo1, logo2;
		Gtk::ToolItem item_logo_1, item_logo_2;
		Gtk::MenuToolButton open_buttons;
		Gtk::Menu open_menu;
		Gtk::ScrolledWindow m_scrolled_window_text_view;
		Gtk::TextView m_text_view;
		Glib::RefPtr<Gtk::TextBuffer> m_ref_text_buffer_shortest_path;
		
		
		std::string PreOpenFile(const std::string& filename);
		void updateTreeView();
		void updateZoom();
		void updateScale();
		void reloadGraph();
		void resetElementSelected();
		void createTreeView(void* root_node, Gtk::TreeModel::Row& row);
		void messageBoxError(const std::string& msg);
		void updateNodesWithSound();
		void updateSoundWithNodes();

		
		// pixbuf
		Glib::RefPtr<Gdk::Pixbuf> graph_image_original;
  		Glib::RefPtr<Gdk::Pixbuf> graph_image;
		
		
		// menu
		Glib::RefPtr<Gtk::UIManager> m_refUIManager;
		Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
		
		
		// graph
		HTKGraph htk_graph;
		std::list<GraphElement> graph_elements;
		std::list<GraphElement> selected_elements;
				
		
		// some attributes
		float scale;
		int mouse_x, mouse_y;
		bool display_shortest_path;
		bool display_shortest_path_selected;
		bool file_opened;
		bool is_reloading;
		bool is_CTRL_pressed;
		double sound_begin, sound_end;
		std::string graph_path;
		int sound_pos_begin, sound_pos_end;
		bool mouse_pressed_over_sound_area;
		bool m_animation_started; // animation is launched when a sound is played
		int m_animation_elapsed_time;
		long m_animation_start;
		std::string file_to_load;

		// tree view related stuff
		class ModelColumns : public Gtk::TreeModel::ColumnRecord
		{
			public:

				ModelColumns()
				{ add(col_id); add(col_attrib); }

				Gtk::TreeModelColumn<Glib::ustring> col_id;
				Gtk::TreeModelColumn<Glib::ustring> col_attrib;
		};
		ModelColumns m_Columns;
		Glib::RefPtr<Gtk::TreeStore> m_refTreeModel;
		Glib::RefPtr<Gtk::TreeSelection> refTreeSelection;
		
		
		// audio
		AudioIO audio;
		std::string sound_filename;
		bool is_sound_loaded;
		

};




#endif /* __MAIN_WINDOW__ */

